//-----------------------------------------------------
//File: lcd_4bit.c
//Auth: ELECTRONIC ASSEMBLY JM
//DATE: 08-28-2013
//-----------------------------------------------------

//---Includes---
#include "sfr_r825.h"
#include "define.h"
#include "lcd_4bit.h"

#include "timer.h"

//--- module global varibles ---


//--- waitstates for testing ---
//use these macros to slow the communication down, it's necessary with this uC, because ports of uC are too slow
#define WAITST 20

//#define Wait(x)
//#define Wait(x) asm("nop")

void Wait (unsigned char waitstates)
{
	unsigned char i=0;
	for (i=0; i<waitstates; i++)
	{
		asm("nop");
	}
}



//-----------------------------------------------------
//Func: initDispl()
//Desc: inits Display
//-----------------------------------------------------
void initDispl(void)
{
	unsigned char i;

	//Port init (dd -> datadirection register; 1 output, 0 input)
	EN = RS = RW = 0; //All pins to high
	DATA = 0x0F;
	ddEN  = 1;
	ddRW = 1;
	ddRS = 1;
	prc2 = 1;  //remove protection of port P0 to change port direction
	ddDATA = 0xF0;
	pur0=0x01; //low 4bits are pulled up (not used)

	RS = 0;
	RW = 0;

	//Reset of LCD-Module is recommended,
	//can't show example, because no pins on controller left
	//-->Reset is pulled high in this example

	//Make sure to get in 4-Bit Mode
	WriteNibble(0x30);
	Wait(WAITST);
	WriteNibble(0x30);
	Wait(WAITST);
	WriteNibble(0x30);
	Wait(WAITST);
	WriteNibble(0x20);

	Delay(1);

	//init Display
	WriteIns(0x2A);	//4-Bit data length extension Bit RE=1; REV=0
	WriteIns(0x09);	//4 line display
	WriteIns(0x06);	//Bottom view
	WriteIns(0x1E);	//Bias setting BS1=1
	WriteIns(0x29);	//4-Bit data length extension Bit RE=0; IS=1
	WriteIns(0x1B);	//BS0=1 -> Bias=1/6
	WriteIns(0x6E); //Devider on and set value
	WriteIns(0x57); //Booster on and set contrast (BB1=C5, DB0=C4)
	WriteIns(0x72); //Set contrast (DB3-DB0=C3-C0)
	WriteIns(0x28); //4-Bit data length extension Bit RE=0; IS=0

	ClrDisplay();
	DisplayOnOff(DISPLAY_ON | CURSOR_ON | BLINK_ON);
}

//-----------------------------------------------------
//Func: WriteChar(character)
//Desc: sends a single character to display
//-----------------------------------------------------
void WriteChar (char character)
{
	WriteData(character);
}

//-----------------------------------------------------
//Func: WriteString(string)
//Desc: sends a string to display, must be 0 terminated
//-----------------------------------------------------
void WriteString(_far char * string)
{
	do
	{
		WriteData(*string++);
	}
	while(*string);
}


//-----------------------------------------------------
//Func: SetPostion(postion)
//Desc: set postion
//-----------------------------------------------------
void SetPostion(char pos)
{
	WriteIns(LCD_HOME_L1+pos);
}

//-----------------------------------------------------
//Func: DisplayOnOff(control)
//Desc: use definitions of header file to set display
//-----------------------------------------------------
void DisplayOnOff(char data)
{
	WriteIns(0x08+data);
}

//-----------------------------------------------------
//Func: DefineCharacter(memory postion, character data)
//Desc: stores an own defined character
//-----------------------------------------------------
void DefineCharacter(unsigned char postion, unsigned char *data)
{
	unsigned char i=0;
	WriteIns(0x40+8*postion);

	for (i=0; i<8; i++)
	{
		WriteData(data[i]);
	}
	SetPostion(LINE1);
}
//-----------------------------------------------------
//Func: ClrDisplay
//Desc: Clears entire Display content and set home pos
//-----------------------------------------------------
void ClrDisplay(void)
{
	WriteIns(0x01);
	SetPostion(LINE1);
}

//-----------------------------------------------------
//Func: SetContrast
//Desc: Sets contrast 0..63
//-----------------------------------------------------
void SetContrast(unsigned char contr)
{
	WriteIns(0x29);
	WriteIns(0x54 | (contr >> 4));
	WriteIns(0x70 | (contr & 0x0F));
	WriteIns(0x28);

}

//-----------------------------------------------------
//Func: SetROM
//Desc: Changes the Rom code (ROMA=0x00, ROMB=0x04, ROMC=0x0C)
//---------------------------------------------------
void SetROM (unsigned char rom)
{
	WriteIns(0x2A);
	WriteIns(0x72);
	WriteData(rom);
	WriteIns(0x28);
}

//-----------------------------------------------------
//Func: SetView
//Desc: view bottom view(0x06), top view (0x05)
//-----------------------------------------------------
void SetView(unsigned char view)
{
	WriteIns(0x2A);
	WriteIns(view);
	WriteIns(0x28);
}

//-----------------------------------------------------
//Func: WriteIns(instruction)
//Desc: sends instruction to display
//-----------------------------------------------------
static void WriteIns(char ins)
{
	CheckBusy();
	RS = 0;
	RW = 0;
	WriteNibble(ins); 	//High nibble
	Wait(WAITST);
	WriteNibble(ins<<4);//low nibble
}

//-----------------------------------------------------
//Func: WriteData(data)
//Desc: sends data to display
//-----------------------------------------------------
static void WriteData(char data)
{
	CheckBusy();
	RS = 1;
	RW = 0;
	WriteNibble(data); 	//High nibble
	Wait(WAITST);
	WriteNibble(data<<4);//low nibble

}

//-----------------------------------------------------
//Func: WriteNibble(data)
//Desc: sends 4 bit to display, use high 4 bits of Byte
//-----------------------------------------------------
static void WriteNibble(char data)
{
	DATA = data|0x0F;

	EN = 1;
	Wait(WAITST/2);
	EN = 0;
}

//-----------------------------------------------------
//Func: CheckBusy()
//Desc: checks if display is idle
//-----------------------------------------------------
unsigned char CheckBusy(void)
{
	unsigned char readData = 1;

	prc2 = 1;
	ddDATA = 0x00; //Data is input
	RS = 0;
	RW = 1;
	Wait(WAITST);
	do
	{
		EN = 1;
		Wait(WAITST/2);
		EN = 0;
		readData = DATA&0xF0;
		Wait(WAITST);

		EN = 1;
		Wait(WAITST/2);
		EN = 0;
		readData |= DATA>>4;
		Wait(WAITST);


	}while(readData&0x80); //check for busyflag

	prc2 = 1;      //remove protection of port P0 to change port direction
	ddDATA = 0xF0; //data is output
	return readData;
}




